﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GazeWebSocketServer.DSP
{
    class FIRfilt
    {
        public Double[] Coefficients {
            get => coefficients;
            private set {
                coefficients = value;
                Initialize();
            }
        }
        public Boolean OutputValid { get; private set; }
        public Double? Output {
            get {
                if (OutputValid)
                {
                    return output;
                }
                else
                {
                    return null;
                }
            }
        }

        private Double output;
        private Double[] ringBuffer;
        private Int64 currentValueIndex;
        private Double[] coefficients;

        public FIRfilt(Double[] coefficients)
        {
            Coefficients = coefficients;
        }

        public static FIRfilt[] createFilterArray(Double[] coefficients, Int64 numberOfFilters)
        {
            FIRfilt[] firFilts = new FIRfilt[numberOfFilters];
            for(Int64 index= 0; index < firFilts.Length; index++)
            {
                firFilts[index] = new FIRfilt(coefficients);
            }
            return firFilts;
        }

        private void Initialize()
        {
            ringBuffer = new Double[Coefficients.Length];
            for (Int64 index = 0; index < ringBuffer.Length; index++)
            {
                ringBuffer[index] = 0.0;
            }

            currentValueIndex = 0;
            output = 0.0;
            OutputValid = false;
        }

        public void Input(Double input)
        {
            ringBuffer[currentValueIndex] = input;
            output = 0.0;
            for (Int64 index = 0; index < ringBuffer.Length; index++)//parallelizable
            {
                output += ringBuffer[(currentValueIndex + index) % Coefficients.Length] * Coefficients[index];
            }
            
            currentValueIndex = (currentValueIndex + 1) % Coefficients.Length;
            if(!OutputValid && currentValueIndex == 0)
            {
                OutputValid = true;
            }
        }
    }
}
